'''This class defines the output of the job and checks if exist a corresponding dataset into DB.'''

import os

from GangaCore.Core.exceptions import (GangaException,
                                   ApplicationConfigurationError)
from GangaCore.GPIDev.Schema import *
from GangaCore.GPIDev.Lib.Dataset import Dataset
from GangaCore.GPIDev.Lib.File import File
from GangaCore.Utility.Config import *
import GangaCore.Utility.logging

import db
import SBDatasetManager
import SBInputDataset
import utils


class SBOutputDataset(Dataset):
    '''Provides methods to set and verify output informations (like output dataset, output files name, output site, etc..).
    The output dataset(s) must exist before submission(it can be created through SBDatasetManager), this is possible because 
    it's known what kind of file the job will be create.'''
    
    _name = "SBOutputDataset"
    _category = "datasets"
    _exportmethods = ["check",
                      "setOutputDataset",]
    _schema = Schema( Version( 1, 0 ), {
        "pairs" : SimpleItem(defvalue={}, hidden=0, protected=0, copyable=1, doc="pattern/dataset_id association"),
    })
    
    def check(self):
        '''This method validates output files informations at submission phase'''
        if len(self.pairs) == 0:
            raise ApplicationConfigurationError(None, 'output dataset pairs cannot be empty')
        
        for key, value in self.pairs.items():
            kwargs = dict()
            kwargs['dataset_id'] = value
            kwargs['owner'] = utils.getOwner()
            
            manager = SBDatasetManager.SBDatasetManager()
            datasets = manager.getDataset(**kwargs)
            
            # only one dataset
            if len(datasets) == 0:
                msg = 'Output dataset %s not found' % value
                raise ApplicationConfigurationError(None, msg)
            assert len(datasets) == 1, 'Dataset consistency error'
            dataset = datasets[0]
            
            # owner
            if dataset['owner'] != utils.getOwner():
                msg = 'You are not the owner of the output dataset %s' % value
                raise ApplicationConfigurationError(None, msg)
            
            # status
            if dataset['status'] not in ['open', 'prepared']:
                msg = 'Output dataset %s status is not open or prepared' % value
                raise ApplicationConfigurationError(None, msg)
            
            # site
            sql = 'SELECT site FROM analysis_dataset_site WHERE dataset_id = %s'
            site = db.read(sql, (r'\x' + value, ))
            if site[0]['site'] != getConfig('SuperB')['submission_site']:
                msg = 'Output site mismatching: the submission site for dataset %s has to be %s' % (value, dataset['site'])
                raise ApplicationConfigurationError(None, msg)
            
            # session
            j = self.getJobObject()
            if isinstance(j.inputdata, SBInputDataset.SBInputPersonalProduction):
                if j.inputdata.session == 'FullSim' and dataset['session'] != 'fullsim':
                    msg = 'Output dataset type should be \'fullsim\''
                    raise ApplicationConfigurationError(None, msg)
                
                if j.inputdata.session == 'FastSim' and dataset['session'] != 'fastsim':
                    msg = 'Output dataset type should be \'fastsim\''
                    raise ApplicationConfigurationError(None, msg)
            else:
                if dataset['session'] != 'analysis':
                    msg = 'Output dataset type should be \'analysis\''
                    raise ApplicationConfigurationError(None, msg)
                
                # parent: exists only for analysis session
                if j.inputdata.dataset_id is None:
                    msg = 'Input dataset is not defined'
                    raise ApplicationConfigurationError(None, msg)
                else:
                    parent_dataset = j.inputdata.dataset_id
                if 'parent' not in dataset['parameters']:
                    sql = 'UPDATE analysis_dataset SET parameters = parameters || %s WHERE dataset_id = %s'
                    db.write(sql, ({'parent': parent_dataset}, r'\x' + value))
                elif dataset['parameters']['parent'] != parent_dataset:
                    msg = 'Input dataset must be %s' % dataset['parameters']['parent']
                    raise ApplicationConfigurationError(None, msg)
    
    def setOutputDataset(self, **kwargs):
        '''Through this method it is possible to define the partern of output files (like *.root) and the corresponding output dataset. 
        To choose the desired dataset, a list of all datasets of the chosen session is printed.'''
        
        key = raw_input('Enter a pattern (eg. *.root): ')
        
        j = self.getJobObject()
        
        if isinstance(j.inputdata, SBInputDataset.SBInputPersonalProduction):
            if j.inputdata.session == 'FastSim':
                kwargs['session'] = 'fastsim'
            elif j.inputdata.session == 'FullSim':
                kwargs['session'] = 'fullsim'
            else:
                raise GangaException('j.inputdata.session is \'%s\'. It must be \'FastSim\' or \'FullSim\'' % j.inputdata.session)
        else:
            kwargs['session'] = 'analysis'
        
        kwargs['owner'] = utils.getOwner()
        kwargs['status'] = ['open', 'prepared']
        
        manager = SBDatasetManager.SBDatasetManager()
        datasets = manager.getDataset(**kwargs)
        dataset = manager.printDatasets(datasets) # print dataset and choose one of them
        
        self.pairs[key] = dataset['dataset_id']

logger = GangaCore.Utility.logging.getLogger()



from __future__ import print_function

import os
import shutil

import pytest

from GangaCore.testlib.GangaUnitTest import start_ganga, stop_ganga, _getGangaPath, ganga_test_dir_name

def gangadir(request):
    """
    Return the gangdir that should be used for a given test.

    Args:
        request: A pytest FixtureRequest object

    Returns:
        str: the name of the gangadir directory to use

    """
    return os.path.join(_getGangaPath(), ganga_test_dir_name, request.module.__name__)


@pytest.yield_fixture(scope='module')
def wipe_gangadir(request):
    """
    Wipe the gangadir for the test at the beginning of the test
    """
    shutil.rmtree(gangadir(request), ignore_errors=True)
    yield


@pytest.yield_fixture(scope='function')
def gpi(request, wipe_gangadir):
    """
    Provide the ganga GPI and runtime to the test.

    This allows full Ganga integration tests. The GPI namespace is
    available via the ``ganga`` fixture argument or via an import of
    ``GangaCore.GPI`` inside the function as usual.
    """
    config_values = getattr(request._pyfuncitem._obj, '_config_values', {})
    config_values = [(k[0], k[1], v) for k, v in config_values.items()]  # Convert from dict to a list of tuples

    start_ganga(gangadir(request), extra_opts=config_values)

    import GangaCore.GPI
    yield GangaCore.GPI

    stop_ganga()



###############################################################################
# Ganga Project. http://cern.ch/ganga
#
# $Id: Classify.py,v 1.1 2008-10-04 17:42:38 karl Exp $
###############################################################################
# File: Classify.py
# Author: K. Harrison
# Created: 070122

"""Module containing class for Classify application"""

__author__  = "K.Harrison <Harrison@hep.phy.cam.ac.uk>"
__date__    = "30 November 2007"
__version__ = "1.2"

import os
import shutil

from GangaCore.Core.exceptions import ApplicationConfigurationError
from GangaCore.GPIDev.Adapters.IApplication import IApplication
from GangaCore.GPIDev.Schema import ComponentItem, Schema, SimpleItem, Version
from GangaCore.Utility import logging
from GangaCore.Utility.files import fullpath

logger = logging.getLogger()

class Classify( IApplication ):

   _schema = Schema( Version( 1, 0 ), {
      "version" : SimpleItem( defvalue = "2.0.0", doc = \
         "String giving version number for Grid software installations" ),
      "libList" : SimpleItem( defvalue = [], doc = \
         "List of paths to java libraries used in classification" ),
      "classifierDir" : SimpleItem( defvalue = "", doc = \
         "Directory containing classifier files" ),
      "outDir" : SimpleItem( defvalue = "", doc = \
         "\n".join( [ "Directory for output files", \
         "If null, use top-level directory of input images" ] ) ),
      "imageList" : SimpleItem( defvalue = "", doc = \
         "Path to file containing list of input images" ),
      "maxImage" : SimpleItem( defvalue = 0, doc = \
         "Maximum number of images to process, no maximum for negative value" ),
      } )

   _category = 'applications'
   _name = 'Classify'

   def __init__( self ):
      super( Classify, self ).__init__()
        
   def master_configure( self ):

      listFile = fullpath( self.imageList )
      if not os.path.isfile( listFile ):
         raise ApplicationConfigurationError\
            ( None, "imageList file = '%s' not found" % self.imageList )

      imageFile = open( listFile )
      imageDir = imageFile.readline().strip().rstrip( os.sep )
      inputList = []

      if imageDir: 
         imageDir = fullpath( imageDir )
         inputList = imageFile.readlines()
         imageFile.close()
         outDir = fullpath( self.outDir )
         if not outDir:
            outDir = imageDir
         tagFile = ""
      else:
         outDir = fullpath( self.outDir )
         tagFile = listFile

      if not outDir:
         raise ApplicationConfigurationError\
            ( None, "Output directory not defined" )

      if not os.path.isdir( outDir ):
         os.makedirs( outDir )

      imageList = []
      for input in inputList:
         url = input.split()[ 0 ]
         relativePath = input[ len( url ) : ].strip().strip( os.sep )
         imageList.append( ( url, relativePath ) )

         if len( imageList ) >= self.maxImage:
            if abs( self.maxImage ) == self.maxImage:
               break

      appDict = \
         {
         "version" : self.version,
         "classifierDir" : fullpath( self.classifierDir ),
         "libList" : [],
         "imageDir" : imageDir,
         "outDir" : outDir,
         "imageList" : imageList,
         "tagFile" : tagFile
         }
      for lib in self.libList:
         appDict[ "libList" ].append( fullpath( lib ) )

      return( False, appDict )

   def configure( self, master_appconfig ):    

      return ( False, master_appconfig )

   def postprocess( self ):
      job = self._getParent()
      try:
         outputdata = job.outputdata
         datatype = outputdata._name
      except AttributeError:
         outputdata = None
         datatype = None

      if datatype:
         outputdata.fill()

      return None



from __future__ import absolute_import
import os
import sys
import mimetypes
import GangaCore.Utility.logging
logger = GangaCore.Utility.logging.getLogger(modulename=True)

from .WNSandbox import OUTPUT_TARBALL_NAME, PYTHON_DIR
from GangaCore.Core.exceptions import GangaException, GangaIOError


class SandboxError(GangaException):

    def __init__(self, message=''):
        GangaException.__init__(self, message)
        self.message = message

    def __str__(self):
        return "SandboxError: %s " % (self.message)


# FIXME: os.system error handling missing in this module!

def createPackedInputSandbox(sandbox_files, inws, name):
    """Put all sandbox_files into tarball called name and write it into to the input workspace.
       This function is called by Ganga client at the submission time.
       Arguments:
                'sandbox_files': a list of File or FileBuffer objects.
                'inws': a InputFileWorkspace object
       Return: a list containing a path to the tarball
       """

#    from GangaCore.Core import FileWorkspace
#    from GangaCore.GPIDev.Lib.File import File

#    tgzfile = os.path.join(tmpdir,name)

    tgzfile = inws.getPath(name)

    import tarfile
    import stat

    logger.debug("Creating packed Sandbox with %s many sandbox files." % len(sandbox_files))

#
# Curent release with os module
#

#   wsdir = os.path.join(tmpdir,"ws")
#   ws = FileWorkspace.FileWorkspace(wsdir)
#   ws.create()
#   for f in sandbox_files:
#       ws.writefile(f)

    # if os.system("tar -C %s -czf %s ."%(wsdir,tgzfile)) !=0:
    #       print "ERROR:: can't create tarball file with InputSandbox"

#
# Future release with tarball module

    if mimetypes.guess_type(tgzfile)[1] in ['gzip']:
        file_format = 'gz'
    elif mimetypes.guess_type(tgzfile)[1] in ['bzip2']:
        file_format = 'bz2'
    else:
        file_format = ''

    with open(tgzfile, 'w:%s' % file_format) as this_tarfile:
        tf = tarfile.open(name=tgzfile, fileobj=this_tarfile, mode="w:gz")
        tf.dereference = True  # --not needed in Windows

        from GangaCore.GPIDev.Lib.File.FileBuffer import FileBuffer
        from GangaCore.GPIDev.Lib.File.File import File
        from GangaCore.GPIDev.Base.Proxy import isType

        for f in sandbox_files:
            fileobj = None
            if isType(f, FileBuffer):
                contents = f.getContents()   # is it FileBuffer?
                # print "Getting FileBuffer Contents"

                from StringIO import StringIO
                fileobj = StringIO(contents)

                tinfo = tarfile.TarInfo()
                # FIX for Ganga/test/Internals/FileBuffer_Sandbox
                # Don't keep the './' on files as looking for an exact filename
                # afterwards won't work
                if f.subdir == os.curdir:
                    tinfo.name = os.path.basename(f.name)
                else:
                    tinfo.name = os.path.join(f.subdir, os.path.basename(f.name))
                import time
                tinfo.mtime = time.time()
                tinfo.size = fileobj.len

            else:
                #   except AttributeError as err:         # File
                # print "Getting File %s" % f.name
                # tf.add(f.name,os.path.join(f.subdir,os.path.basename(f.name)))
                logger.debug("Opening file for sandbox: %s" % f.name)
                try:
                    fileobj = open(f.name)
                except Exception as err:
                    raise SandboxError("File '%s' does not exist." % f.name)

                tinfo = tf.gettarinfo(f.name, os.path.join(f.subdir, os.path.basename(f.name)))

            if f.isExecutable():
                tinfo.mode = tinfo.mode | stat.S_IXUSR
            tf.addfile(tinfo, fileobj)
            fileobj.close()
        tf.close()

    return [tgzfile]


def createInputSandbox(sandbox_files, inws):
    """Put all sandbox_files into the input workspace.
       This function is called by Ganga client at the submission time.
       Arguments:
                'sandbox_files': a list of File or FileBuffer objects.
                'inws': a InputFileWorkspace object
       Return: a list of paths to sanbdox files in the input workspace
    """

    #    from GangaCore.Core import FileWorkspace

    return [inws.writefile(f, f.isExecutable()) for f in sandbox_files]


def getPackedOutputSandbox(src_dir, dest_dir):
    """Unpack output files from tarball in source directory and
       write them to destination directory
       This function is called by Ganga client at the completing of job.
       Complementary to createPacedOutput()
       Arguments:
                'src_dir': source directory with tarball
                'dest_dir': desti nation directory for output files
    """

    tgzfile = os.path.join(src_dir, OUTPUT_TARBALL_NAME)
    if os.access(tgzfile, os.F_OK):

        import tarfile

        try:
            tf = tarfile.open(tgzfile, "r:*")
        except tarfile.ReadError:
            logger.warning('Sandbox is empty or unreadable')
            return
        else:
            [tf.extractall(dest_dir, [tarinfo]) for tarinfo in tf]
            tf.close()


#####################################################



from __future__ import absolute_import

import threading
from functools import wraps

import GangaCore.Utility.logging

from .CredentialStore import credential_store, needed_credentials, get_needed_credentials
from GangaCore.Core.exceptions import CredentialsError, InvalidCredentialError

logger = GangaCore.Utility.logging.getLogger()


def require_credential(method):
    """
    A decorator for labelling a method as needing a credential.

    If the decorated method is called directly by the user (checked by looking which thread we are on) then if a
    credential is not available, a prompt is given to request one. If the method is being called in another thread
    (monitoring or similar) then a ``CredentialsError`` is raised.

    Uses the method's object's ``credential_requirements`` attribute
    """
    @wraps(method)
    def cred_wrapped_method(self, *args, **kwargs):
        cred_req = self.credential_requirements

        try:
            cred = credential_store[cred_req]
        except KeyError:
            if isinstance(threading.current_thread(), threading._MainThread):  # threading.main_thread() in Python 3.4
                logger.warning('Required credential [%s] not found in store', cred_req)
                cred = credential_store.create(cred_req, create=True)
            else:
                raise CredentialsError('Cannot get proxy which matches requirements {0}'.format(cred_req))

        if not cred.is_valid():
            if isinstance(threading.current_thread(), threading._MainThread):  # threading.main_thread() in Python 3.4
                logger.info('Found credential [%s] but it is invalid. Trying to renew...', cred)
                cred.renew()
            else:
                raise InvalidCredentialError('Proxy is invalid')

        return method(self, *args, **kwargs)
    return cred_wrapped_method





#
# Dictionary with all parameters available for the
# section CRAB.
#
# 08/06/10 @ ubeda
#

from GangaCore.GPIDev.Base import GangaObject
from GangaCore.GPIDev.Schema import *

class CRAB(GangaObject):

    _comments = []
    _comments.append('http://belforte.home.cern.ch/belforte/misc/test/crab-v2.7.2.html#jobtype__')
    _comments.append('http://belforte.home.cern.ch/belforte/misc/test/crab-v2.7.2.html#server_name') 
    _comments.append('http://belforte.home.cern.ch/belforte/misc/test/crab-v2.7.2.html#use_server')
    _comments.append('http://belforte.home.cern.ch/belforte/misc/test/crab-v2.7.2.html#scheduler')
    _comments.append('--- TO DO ---')

    schemadic = {}
    schemadic['jobtype']     = SimpleItem(defvalue=None, typelist=['type(None)','str'], doc=_comments[0])
    schemadic['server_name'] = SimpleItem(defvalue=None, typelist=['type(None)','str'], doc=_comments[1])
    schemadic['use_server']  = SimpleItem(defvalue=None, typelist=['type(None)','int'], doc=_comments[2])
    schemadic['scheduler']   = SimpleItem(defvalue=None, typelist=['type(None)','str'], doc=_comments[3])
    schemadic['submit_host'] = SimpleItem(defvalue=None, typelist=['type(None)','str'], doc=_comments[4])


    _schema =  Schema(Version(0,0), {})
    _hidden = 1



#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#
import os
import tempfile
from GangaCore.Core.exceptions import ApplicationConfigurationError
import GangaCore.Utility.Config
from GangaCore.Utility.files import expandfilename
from GangaCore.GPIDev.Lib.File import FileBuffer, File
import GangaCore.Utility.logging
from GangaDirac.Lib.RTHandlers.DiracRTHUtils import diracAPI_script_template, dirac_outputfile_jdl
from GangaCore.GPIDev.Base.Proxy import isType
from GangaGaudi.Lib.Applications.Gaudi import Gaudi
logger = GangaCore.Utility.logging.getLogger()
#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#


def jobid_as_string(job):
    """ Return the string representation of the job, should be same as job.getFQID() """
    jstr = ''
    if job.master:
        jstr = str(job.master.id) + os.sep + str(job.id)
    else:
        jstr = str(job.id)
    return jstr

def lhcbdirac_outputfile_jdl(output_files):
    """ Construct the setOutputFile section of the JDL for this job
    Args:
        output_files (list): A list of DiracFile objects to have their output injected into the dirac submit script
    """

    DiracScript = dirac_outputfile_jdl(output_files, False)

    DiracScript = DiracScript.replace('###OUTPUT_SE###', '###OUTPUT_SE###,replicate=\'###REPLICATE###\'')

    DiracScript = DiracScript.replace('outputPath', 'OutputPath').replace('outputSE', 'OutputSE')

    return DiracScript

def lhcbdiracAPI_script_template():
    """ Create a template LHCbDirac LHCbJob by taking the Dirac Job and making some modifications to teh template"""

    DiracScript = diracAPI_script_template()

    DiracLHCb_Options = """
j.setApplicationScript('###APP_NAME###', '###APP_VERSION###', '###APP_SCRIPT###', logFile='###APP_LOG_FILE###', systemConfig='###PLATFORM###')
j.setAncestorDepth(###ANCESTOR_DEPTH###)
"""


    DiracScript = DiracScript.replace('\'###EXE_LOG_FILE###\'', '\'###EXE_LOG_FILE###\', systemConfig=\'###PLATFORM###\'')
    DiracScript = DiracScript.replace('j.setPlatform( \'ANY\' )', 'j.setDIRACPlatform()')

    setName_str = 'j.setName(\'###NAME###\')'
    DiracScript = DiracScript.replace(setName_str, "%s\n%s" % (setName_str, DiracLHCb_Options))

    return DiracScript


def is_gaudi_child(app):
    """ Does this app inherit from Gaudi True/False"""
    if isType(app, Gaudi):
        return True
    return False


class filenameFilter:
    """ class for returning a given filename as a filter"""
    def __init__(self, filename):
        self.filename = filename

    def __call__(self, file):
        return file.name == self.filename


def getXMLSummaryScript(indent=''):
    '''Returns the necessary script to parse and make sense of the XMLSummary data'''
    import inspect
    from GangaLHCb.Lib.Applications.AppsBaseUtils import activeSummaryItems
    script = "###INDENT#### Parsed XMLSummary data extraction methods\n"

    for summaryItem in activeSummaryItems().values():
        script += ''.join(['###INDENT###' + line for line in inspect.getsourcelines(summaryItem)[0]])
    script += ''.join(['###INDENT###' + line for line in inspect.getsourcelines(activeSummaryItems)[0]])

    import inspect
    script_location = os.path.join(os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))),
                                   'XMLWorkerScript.py.template')

    from GangaCore.GPIDev.Lib.File import FileUtils
    xml_script = FileUtils.loadScript(script_location, '###INDENT###')

    script += xml_script

    return script.replace('###INDENT###', indent)


def create_runscript(useCmake=False):
    """ Returna run script for Gaudi Apps """
    from GangaLHCb.Lib.Applications.EnvironFunctions import construct_run_environ
    environ_script = construct_run_environ(useCmake)

    import inspect
    script_location = os.path.join(os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))),
                                                   'WorkerScript.py.template')

    from GangaCore.GPIDev.Lib.File import FileUtils
    worker_script = FileUtils.loadScript(script_location, '')

    worker_script = worker_script.replace('###CONSTRUCT_ENVIRON###', environ_script)

    return worker_script

#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#



from GangaCore.GPIDev.Base import GangaObject
from GangaCore.GPIDev.Schema import Schema, Version
from GangaCore.Utility.logging import getLogger
from GangaCMS.Lib.CRABTools.CRABServerError import CRABServerError

import datetime
import os
import shlex
import shutil
import subprocess
import time


logger = getLogger()


class CRABServer(GangaObject):
    """Helper class to launch CRAB commands."""
    _schema = Schema(Version(0, 0), {})
    _hidden = True

    def _send(self, cmd, operation, env):
        """Launches a command and waits for output."""
        try:
            logger.debug('Launching a CRAB command: %s' % cmd)
            init = datetime.datetime.now()
            p = subprocess.Popen(shlex.split(cmd), stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE, bufsize=-1, env=env)
            stdout, stderr = p.communicate()
            p.wait()
            end = datetime.datetime.now()
            logger.debug('Finished CRAB command: %s' % cmd)
            logger.info('%s took %d sec.' % (operation, (end - init).seconds))

            if p.returncode != 0:
                raise CRABServerError('CRAB %s returned %s' % (operation,
                                                               p.returncode))
        except OSError as e:
            logger.error(stdout)
            logger.error(stderr)
            raise CRABServerError(e, 'OSError %s crab job(s).' % operation)

    def _send_with_retry(self, cmd, operation, env, retries=3, delay=60):
        """Wrapper to add some retries to the CRAB command launching."""
        assert retries > 0
        assert delay >= 0

        for _ in range(retries):
            try:
                self._send(cmd, operation, env)
                return
            except CRABServerError:
                time.sleep(delay)

        raise CRABServerError('CRAB %s failed %d times' % (operation, retries))

    def create(self, job):
        """Create a new CRAB jobset."""
        cfgfile = os.path.join(job.inputdir, 'crab.cfg')
        if not os.path.isfile(cfgfile):
            raise CRABServerError('File "%s" not found.' % cfgfile)

        # Clean up the working dir for the CRAB UI.
        shutil.rmtree(job.inputdata.ui_working_dir, ignore_errors=True)

        cmd = 'crab -create -cfg %s' % cfgfile
        self._send_with_retry(cmd, 'create', job.backend.crab_env)
        return True

    def submit(self, job):
        """Submit an already created jobset."""
        if not os.path.exists(job.inputdata.ui_working_dir):
            raise CRABServerError('Workdir "%s" not found.' %
                                  job.inputdata.ui_working_dir)

        cmd = 'crab -submit -c %s' % job.inputdata.ui_working_dir
        self._send_with_retry(cmd, 'submit', job.backend.crab_env)
        return True

    def status(self, job):
        """Get the status of a jobset."""
        if not os.path.exists(job.inputdata.ui_working_dir):
            raise CRABServerError('Workdir "%s" not found.' %
                                  job.inputdata.ui_working_dir)

        cmd = 'crab -status -c %s' % job.inputdata.ui_working_dir
        self._send_with_retry(cmd, 'status', job.backend.crab_env)
        return True

    def kill(self, job):
        """Kill all the jobs on the task."""
        if not os.path.exists(job.inputdata.ui_working_dir):
            raise CRABServerError('Workdir "%s" not found.' %
                                  job.inputdata.ui_working_dir)

        if not job.master:
            cmd = 'crab -kill all -c %s' % job.inputdata.ui_working_dir
        else:
            cmd = 'crab -kill %d -c %s' % (int(job.id) + 1,
                                           job.inputdata.ui_working_dir)
        self._send_with_retry(cmd, 'kill', job.backend.crab_env)
        return True

    def resubmit(self, job):
        """Resubmit an already created job."""
        if not os.path.exists(job.inputdata.ui_working_dir):
            raise CRABServerError('Workdir "%s" not found.' %
                                  job.inputdata.ui_working_dir)

        cmd = 'crab -resubmit %d -c %s' % (int(job.id) + 1,
                                           job.inputdata.ui_working_dir)
        self._send_with_retry(cmd, 'resubmit', job.backend.crab_env)
        return True

    def getOutput(self, job):
        """Retrieve the output of the job."""
        if not os.path.exists(job.inputdata.ui_working_dir):
            raise CRABServerError('Workdir "%s" not found.' %
                                  job.inputdata.ui_working_dir)

        cmd = 'crab -getoutput %d -c %s' % (int(job.id) + 1,
                                            job.inputdata.ui_working_dir)
        self._send_with_retry(cmd, 'getoutput', job.backend.crab_env)
        # Make output files coming from the WMS readable.
        for root, _, files in os.walk(os.path.join(job.inputdata.ui_working_dir,
                                                   'res')): # Just 'res'.
            for f in files:
                os.chmod(os.path.join(root, f), 0o644)
        return True

    def postMortem(self, job):
        """Retrieves the postmortem information."""
        if not os.path.exists(job.inputdata.ui_working_dir):
            raise CRABServerError('Workdir %s not found.' %
                                  job.inputdata.ui_working_dir)

        cmd = 'crab -postMortem %d -c %s' % (int(job.id) + 1,
                                             job.inputdata.ui_working_dir)
        self._send_with_retry(cmd, 'postMortem', job.backend.crab_env)
        return True



import re, os, sys, types

from GangaCore.Core.exceptions import BackendError
from GangaCore.GPIDev.Base import GangaObject
from GangaCore.GPIDev.Schema import *
from GangaCore.Utility.Config import getConfig, ConfigError
from GangaCore.Utility.logging import getLogger
from GangaCore.Lib.LCG import LCGRequirements

from dq2.info.TiersOfATLAS import _refreshToACache, ToACache, _resolveSites, getSites 

logger = getLogger()

config = getConfig('Athena')

try:
    excluded_sites = config['ExcludedSites'].split()
except ConfigError:
    excluded_sites = []

slc3_req = '''(( other.GlueHostOperatingSystemName  == "CentOS" || 
      other.GlueHostOperatingSystemName  == "RedHatEnterpriseAS" 
    ) &&
    ( other.GlueHostOperatingSystemRelease >= 3.0 && 
      other.GlueHostOperatingSystemRelease < 4.0 ) 
    ) ||
    ( other.GlueHostOperatingSystemName  == "Scientific Linux" || 
      other.GlueHostOperatingSystemName  == "Scientific Linux CERN"
    )'''

slc4_req = '''( other.GlueHostOperatingSystemName  == "CentOS" ||
     other.GlueHostOperatingSystemName  == "RedHatEnterpriseAS"  ||
     other.GlueHostOperatingSystemName  == "ScientificSL" ||
     other.GlueHostOperatingSystemName  == "ScientificCERNSLC" 
   ) &&
   ( other.GlueHostOperatingSystemRelease >= 4.0 &&
     other.GlueHostOperatingSystemRelease < 5.0 
   )'''

CESEInfo      = None
CESEInfoURL   = 'http://ganga.web.cern.ch/ganga/ATLAS/cese_info.dat.gz'
CESEInfoLocal = '/tmp/ganga.cese_info.dat.gz_%d' % os.getuid()

def _loadCESEInfo():

    import gzip
    import cPickle as pickle

    result = {}
    try:
        input = gzip.open(CESEInfoLocal,'rb')
        result['time']          = pickle.load(input)
        result['ce_info']       = pickle.load(input)
        result['se_info']       = pickle.load(input)
        result['lcg_site_info'] = pickle.load(input)
        try:
            result['blacklist'] = pickle.load(input)
        except EOFError:
            result['blacklist'] = []
        try:
            result['access_info'] = pickle.load(input)
        except EOFError:
            result['access_info'] = []
        try:
            result['blacklist_release_info'] = pickle.load(input)
        except EOFError:
            result['blacklist_release_info'] = []
        try:
            result['creamce_info']  = pickle.load(input)
        except EOFError:
            result['creamce_info']  = []
            
        input.close()
    except Exception:
        logger.error('Cannot read CE-SE association from file.')
        result = None

    return result

def _downloadCESEInfo():

    import urllib, socket, gzip
    from cStringIO import *
    import cPickle as pickle
    from stat import *

#   timeouts are not supported in python 2.2

    try:
        dto = socket.getdefaulttimeout()
        socket.setdefaulttimeout(10)
    except AttributeError:
        pass

    retry = 0
    data = None

    while not data and retry < 3:
        retry += 1
        try:
            data = urllib.urlopen(CESEInfoURL).read()
        except Exception as e:
            logger.warning(e)
            pass

    try:    
        socket.setdefaulttimeout(dto)
    except AttributeError:
        pass

    if not data:
        logger.error('Could not download CE-SE association from the GANGA website.')
        return None

    result = {}
    try:
        input = gzip.GzipFile(fileobj=StringIO(data))
        result['time']          = pickle.load(input)
        result['ce_info']       = pickle.load(input)
        result['se_info']       = pickle.load(input)
        result['lcg_site_info'] = pickle.load(input)
        try:
            result['blacklist'] = pickle.load(input)
        except EOFError:
            result['blacklist'] = []
        try:
            result['access_info'] = pickle.load(input)
        except EOFError:
            result['access_info'] = []
        try:
            result['blacklist_release_info'] = pickle.load(input)
        except EOFError:
            result['blacklist_release_info'] = []
        try:
            result['creamce_info']  = pickle.load(input)
        except EOFError:
            result['creamce_info']  = []
        input.close()
    except Exception as e:
        logger.error(e)
        logger.error('Cannot read CE-SE association downloaded from the GANGA website..')
        return None
    
    local = open(CESEInfoLocal,'wb')
    local.write(data)
    local.close()
    os.chmod(CESEInfoLocal,S_IRUSR | S_IWUSR)

    return result

def _refreshCESEInfo():
    '''Refresh CE-SE association information'''

    import time
    from stat import *

    global CESEInfo

    if CESEInfo and time.time() - CESEInfo['time'] < 3600: return

    CESEInfo = None

    if os.path.exists(CESEInfoLocal):
        ctime = os.stat(CESEInfoLocal)[ST_CTIME]
        if time.time() - ctime < 3600:
            logger.info('Reading local copy of CE-SE association file.')
            CESEInfo = _loadCESEInfo()
            if not CESEInfo:
                logger.debug('CE-SE association file is removed and a new copy will be downloaded.')
                os.unlink(CESEInfoLocal)
    
    retry = 0
    while not CESEInfo and retry < 3:
        retry += 1
        logger.info('Downloading remote copy of CE-SE association file.')
        CESEInfo = _downloadCESEInfo()
  
    if not CESEInfo:
        logger.warning('CE-SE association could not be downloaded. Using the old file')
        CESEInfo = _loadCESEInfo()
        if not CESEInfo:
            logger.debug('CE-SE association file is not usable.')
            os.unlink(CESEInfoLocal)

    else:
#       just warn if the file is older then one day
        if time.time() - CESEInfo['time'] > 3600*24:
            logger.error('CE-SE associations are stale. Please report the issue to the mailing list.')
        CESEInfo['time'] = time.time()
    
def getSEsForSites(ids):
    '''Retrieve the SEs for a site'''

    _refreshToACache()

#   sites can be a colon seperated list like 'CERN:LYON:ASGC'

    re_srm = re.compile('srm://([^/]+)(/.+)')
    se_dict = {}
    for id in ids:

        sites = _resolveSites(id.upper())
        if not sites:
            logger.error('Site %s not found in TiersOfATLAS',id)
            continue

        for site in sites:
            site_info = ToACache.sites[site]
            if 'srm' not in site_info: 
                logger.error('Site %s has no srm info in TiersOfATLAS',site)
                continue
            sitesrm = site_info['srm']
            if sitesrm == '': 
                logger.debug('Site %s has no srm info in TiersOfATLAS',site)
                continue
            sitesrm = re.sub('token:*\w*:','', sitesrm)
            sitesrm = re.sub(':*\d*/srm/managerv2\?SFN=','', sitesrm)
            match = re_srm.match(sitesrm)
            if not match:
                logger.debug('Cannot extract host from %',sitesrm)
                continue
            se_dict[match.group(1)] = True

    return se_dict.keys()

def getCEsForSites(ids, excluded_ids = [], CREAM = False, cputime=0 ):
    '''Retrieve the CEs for a site'''

    _refreshToACache()
    _refreshCESEInfo()

    re_srm = re.compile('srm://([^/]+)(/.+)')
    ce_dict = {}
    for id in ids:

#       allow the full use of all ToA names as FZKSITES etc.
 
        sites = _resolveSites([id.upper()])
        if not sites:
            logger.error('Site %s not found in TiersOfATLAS',id)
            continue

        # remove excluded sites
        try:
            excluded_sites = config['ExcludedSites'].split()
        except ConfigError:
            excluded_sites = [ ]

        if excluded_ids:
            excluded_sites = excluded_ids + excluded_sites 
        
        for site in excluded_sites:
            if site in sites:
                logger.warning('Site %s has been excluded.',site)
                sites.remove(site)

#       try to find CEs associated to a site via srm tag and/or associated names tag

        for site in sites:
            site_info = ToACache.sites[site] 
            ces = []
            if 'srm' in site_info:
                sitesrm = site_info['srm']
                if sitesrm == '': 
                    logger.debug('Site %s has no srm info in TiersOfATLAS',site)
                    continue
                sitesrm = re.sub('token:*\w*:','', sitesrm)
                sitesrm = re.sub(':*\d*/srm/managerv2\?SFN=','', sitesrm)
                match = re_srm.match(sitesrm)
                if not match:
                    logger.debug('Cannot extract host from %s',sitesrm)
                else:
                    try:
                        if CREAM:
                            ces = CESEInfo['se_info'][match.group(1)]['close_creamce']
                            if cputime > 0:
                                for ces_tmp in ces:
                                    cescputime = CESEInfo['creamce_info'][ces_tmp]['cputime']
                                    if cescputime:
                                        cescputime = int(cescputime)
                                    if cputime < cescputime:
                                        ces = [ ces_tmp ]
                                        break
                        else:
                            ces = CESEInfo['se_info'][match.group(1)]['close_ce']
                    except KeyError:
                        logger.debug('Did not find CE-SE association for %s',match.group(1))

            if not ces:
                try:
                    lcg_site = site_info['alternateName'][-1].upper()
                    ces = CESEInfo['lcg_site_info'][lcg_site]
                except Exception:
                    logger.debug('No CE information on site %s. Maybe it failes the SAM test.',site)

            for ce in ces: 
                ce_dict[ce] = True
                
    return ce_dict.keys()

def _resolveSites(sites):

    new_sites = []
    for site in sites:
        if site in ToACache.topology:
            new_sites += _resolveSites(ToACache.topology[site])
        else:
            new_sites.append(site)

    return new_sites

def getAllSites(only_lcg=True,resolve=False, excluded_clouds=[], excluded_sites=[], blacklist=True):
    '''list all sites defined in TiersOfATLAS'''

    _refreshToACache()
    _refreshCESEInfo()

    sites = []
    if not 'TO' in excluded_clouds:
        sites += getSites('CERN')
    if not 'IT' in excluded_clouds:
        sites += getSites('ITALYSITES')
    if not 'ES' in excluded_clouds:
        sites += getSites('SPAINSITES')
    if not 'FR' in excluded_clouds:
        sites += getSites('FRANCESITES')
    if not 'UK' in excluded_clouds:
        sites += getSites('UKSITES')
    if not 'DE' in excluded_clouds:
        sites += getSites('FZKSITES')
    if not 'NL' in excluded_clouds:
        sites += getSites('NLSITES')
    if not 'TW' in excluded_clouds:
        sites += getSites('TAIWANSITES')
    if not 'CA' in excluded_clouds:
        sites += getSites('CANADASITES')    

    if not only_lcg:
        sites += getSites('USASITES')
        sites += getSites('NDGF')

    # exclude sites
    for site in excluded_sites:
        if site in sites:
            sites.remove(site)
    
    if resolve: sites = _resolveSites(sites)

    # exclude sites - check again after site resolution
    for site in excluded_sites:
        if site in sites:
            sites.remove(site)

    sites.sort()

    if (blacklist):
        for bad_site in CESEInfo['blacklist']:
            try:
                sites.remove(bad_site)
            except ValueError:
                pass
     
    return sites

def listT2s(name,t1=None):

    t2s = ToACache.topology[name]
    try:
        if t1: t2s.remove(t1)
    except:
        pass

    return t2s

def getCloudInfo():
    '''get cloud information'''

    info =  [ ( 'T0', 'T0', 'CERN') ]
    info += [ ( 'T1', 'IT', 'CNAF' ) ]
    info += [ ( 'T2', 'IT', site ) for site in listT2s('ITALYSITES','CNAF') ]
    info += [ ( 'T1', 'ES', 'PIC' ) ]
    info += [ ( 'T2', 'ES', site ) for site in listT2s('SPAINSITES','PIC') ]
    info += [ ( 'T1', 'FR', 'LYON' ) ]
    info += [ ( 'T2', 'FR', site ) for site in listT2s('FRTIER2S') ]
    info += [ ( 'T1', 'UK', 'RAL' ) ]
    info += [ ( 'T2', 'UK', site ) for site in listT2s('UKTIER2S') ]
    info += [ ( 'T1', 'DE', 'FZK' ) ]
    info += [ ( 'T2', 'DE', site ) for site in listT2s('FZKSITES','FZK') ]
    info += [ ( 'T1', 'NL', 'SARA' ) ]
    info += [ ( 'T2', 'NL', site ) for site in listT2s('NLSITES','SARA') ]
    info += [ ( 'T1', 'TW', 'ASGC' ) ]
    info += [ ( 'T2', 'TW', site ) for site in listT2s('TAIWANSITES','ASGC') ]
    info += [ ( 'T1', 'CA', 'TRIUMF' ) ]
    info += [ ( 'T2', 'CA', site ) for site in listT2s('CANADASITES','TRIUMF') ]
    info += [ ( 'T1', 'US', 'BNL' ) ]
    info += [ ( 'T2', 'US', site ) for site in listT2s('USASITES','BNL') ] 
    info += [ ( 'T1', 'NG', 'NDGF' ) ] 
    
    return info

def getAccessInfo(sitename=''):
    """List prefered input access method of site"""

    _refreshCESEInfo()

    if CESEInfo['access_info'] and sitename in CESEInfo['access_info']:
        return CESEInfo['access_info'][sitename]
    else:
        return []

def getReleaseBlacklistInfo():
    """List release blacklist"""

    _refreshCESEInfo()

    if CESEInfo['blacklist_release_info']:
        return CESEInfo['blacklist_release_info']
    else:
        return []

class AtlasLCGRequirements(LCGRequirements):
    '''LCG requirements for ATLAS.

    See also: JDL Attributes Specification at http://cern.ch/glite/documentation
    '''

    _schema = Schema(Version(1,2), { 
        'software'        : SimpleItem(defvalue = [], typelist=['str'], sequence=1,doc='Software Installations'),
        'nodenumber'      : SimpleItem(defvalue=1,doc='Number of Nodes for MPICH jobs'),
        'memory'          : SimpleItem(defvalue=None, typelist=['type(None)','int'], doc='Mininum available memory (MB)'),
        'cputime'         : SimpleItem(defvalue=None, typelist=['type(None)','int'], doc='Minimum available CPU time (min)'),
        'walltime'        : SimpleItem(defvalue=None, typelist=['type(None)','int'], doc='Mimimum available total time (min)'),
        'ipconnectivity'  : SimpleItem(defvalue=False,doc='External connectivity'),
        'dq2client_version'  : SimpleItem(defvalue='',doc='DQ2 client version on the computing element'),
        'other'           : SimpleItem(defvalue = [], typelist=['str'], sequence=1,doc='Other Requirements'),
        'sites'           : SimpleItem(defvalue = [], typelist=['str'], sequence=1,doc='ATLAS site names'),
        'excluded_sites'  : SimpleItem(defvalue = [], typelist=['str'], sequence=1,doc='ATLAS site names to be excluded'),
        'excluded_clouds' : SimpleItem(defvalue = [], typelist=['str'], sequence=1,doc='ATLAS cloud names to be excluded'),
        'cloud'           : SimpleItem(defvalue = 'ALL', doc='ATLAS cloud name: CERN, IT, ES, FR, UK, DE, NL, TW, CA, US, NG'),
        'anyCloud'        : SimpleItem(defvalue = True, doc='Set to True to allow cross-cloud submission. Use with the \'ALL\' cloud option.'),
        'os'              : SimpleItem(defvalue ='', doc='Operation Systems')
    })

    _category = 'LCGRequirements'
    _name = 'AtlasLCGRequirements'
    _exportmethods = ['list_ce', 'list_se','list_sites','list_clouds', 'list_sites_cloud', 'cloud_from_sites', 'list_access_info', 'list_release_blacklist' ]

    _cloudNameList = { 'T0' : 'CERN',
                       'IT' : 'ITALYSITES',
                       'ES' : 'SPAINSITES',
                       'FR' : 'FRANCESITES',
                       'UK' : 'UKSITES',
                       'DE' : 'FZKSITES',
                       'NL' : 'NLSITES',
                       'TW' : 'TAIWANSITES',
                       'CA' : 'CANADASITES',
                       'US' : 'USASITES',
                       'NG' : 'NDGF'
                       }
    
    _GUIPrefs = [ 
         { 'attribute' : 'software',       'widget' : 'String_List' },
         { 'attribute' : 'nodenumber',     'widget' : 'Int' },
         { 'attribute' : 'memory',         'widget' : 'Int' },
         { 'attribute' : 'cputime',        'widget' : 'Int' },
         { 'attribute' : 'walltime',       'widget' : 'Int' },
         { 'attribute' : 'ipconnectivity', 'widget' : 'Bool' },
         { 'attribute' : 'other',          'widget' : 'String_List' },
         { 'attribute' : 'sites',          'widget' : 'String_List' },
         { 'attribute' : 'cloud',          'widget' : 'String' },
         { 'attribute' : 'excluded_sites', 'widget' : 'String_List' },
         { 'attribute' : 'os',             'widget' : 'String' } 
    ]

    def __init__(self):
      
        super(AtlasLCGRequirements,self).__init__()

    def merge(self,other):
        '''Merge requirements objects'''
      
        if not other: return self
      
        merged = AtlasLCGRequirements()

        for name in self._schema.datadict.keys():
            try:
                attr = getattr(other,name)
            except AttributeError:
                attr = None
            if not attr: attr = getattr(self,name)
            setattr(merged,name,attr)
         
        return merged

    def convert(self):
        '''Convert the condition in a JDL specification'''
      
        requirements = super(AtlasLCGRequirements,self).convert()

        if self.sites:
            ce_requirement = ' ||\n     '.join([ 'other.GlueCEUniqueID == "%s"' % ce for ce in getCEsForSites(self.sites, self.excluded_sites)])
            if not ce_requirement:
                raise BackendError('LCG','Job cannot be submitted as no valid site has been specified.')
            requirements.append('( %s )' % ce_requirement)

        if self.os:
            os_name = self.os.lower()
            if os_name == 'slc3':
                requirements.append(slc3_req)
            elif os_name == 'slc4':
                requirements.append(slc4_req)
            else:
               raise BackendError('LCG','Job cannot be submitted as unknown OS %s has been requested.'%self.os)
            
        return requirements

    def list_se(self,ids):

        if isinstance(ids,str):
            ids = ids.split(':')

        return getSEsForSites(ids)

    def list_ce(self,ids):

        if isinstance(ids,str):
            ids = ids.split(':')

        return getCEsForSites(ids)

    def list_sites(self,only_lcg=True,resolve=False, req_str = ''):

        sites = getAllSites(only_lcg,resolve,self.excluded_clouds,self.excluded_sites)

        if (req_str != '') and (self._name == 'AtlasLCGRequirements'):
            # check release requirements
            old_software = self.software
            self.software = [req_str]
            be = self._getParent()
            matches = be.get_requirement_matches()

            # find the ces for each site
            new_sites = []
            for site in sites:
                ces = getCEsForSites([site])
                for ce in ces:
                    if ce in matches:
                        new_sites.append(site)
                        break

            sites = new_sites
            
        return sites

    def list_clouds(self):

        #return getCloudInfo()
        return self._cloudNameList.keys()

    def list_sites_cloud(self, cloudName='', blacklist=True, req_str = ''):


        if cloudName:
            cloudID = cloudName
        else:
            cloudID = self.cloud

        try:
            cloud = self._cloudNameList[cloudID]
        except:
            cloud = cloudID
           
        sites = getSites(cloud)
        
        # exclude sites
        for site in self.excluded_sites:
            if site in sites:
                sites.remove(site)

        # blacklist
        if (blacklist):
            _refreshCESEInfo()
            for bad_site in CESEInfo['blacklist']:
                try:
                    sites.remove(bad_site)
                except ValueError:
                    pass
                
        if (req_str != '') and (self._name == 'AtlasLCGRequirements'):
            # check release requirements
            old_software = self.software
            self.software = [req_str]
            be = self._getParent()
            matches = be.get_requirement_matches()

            # find the ces for each site
            new_sites = []
            for site in sites:
                ces = getCEsForSites([site])
                for ce in ces:
                    if ce in matches:
                        new_sites.append(site)
                        break

            sites = new_sites
            
        if sites:
            return sites
        raise BackendError('LCG','Could not find any sites for selected cloud %s. Allowed clouds: %s'%(cloud,self._cloudNameList.keys()))

    def cloud_from_sites( self, sites = [] ):
        
        # Return the cloud given the site
        if not sites:
            sites = self.sites

        if sites.__class__.__name__ == 'str':
            sites = [sites]

        if len(sites) == 0:
            return {}
        
        mapping = {}
        
        # sort info for site -> cloud
        for cloud in self.list_clouds():
            cloud_sites = getSites(self._cloudNameList[cloud])
            for site in cloud_sites:
                mapping[ site ] = cloud

        # now output the mapping requested
        output = {}
        for site in sites:
            try:
                output[site] = mapping[site]
            except:
                logger.warning("Could not find cloud associated with site '%s'." % site)
                if site == 'CERN-MOCKTEST':
                    output[site] = 'T0'

        return output
                                               
    def list_access_info(self,sitename=''):

        _refreshToACache()

        if self.sites:
            sitename = self.sites

        if sitename.__class__.__name__ == 'str':
            try:
                site_info = ToACache.sites[sitename]
                alternateName = site_info['alternateName'][-1].upper()
            except:
                alternateName = sitename
            return { alternateName : getAccessInfo(alternateName) }
        else:
            info = {} 
            for site in sitename:
                try:
                    site_info = ToACache.sites[site]
                    alternateName = site_info['alternateName'][-1].upper()
                except:
                    alternateName = site
                info[alternateName] = getAccessInfo(alternateName)
            return info
                
    def list_release_blacklist(self):
        return getReleaseBlacklistInfo()
            

atlascreamrequirements_schema_datadict = AtlasLCGRequirements._schema.inherit_copy().datadict

class AtlasCREAMRequirements(AtlasLCGRequirements):

    atlascreamrequirements_schema_datadict.update( {
        'cputime'         : SimpleItem(defvalue=0, typelist=['type(None)','int'], doc='Minimum available CPU time (min)'),
        } )

    _schema   = Schema(Version(1,2), atlascreamrequirements_schema_datadict)
    _category = 'LCGRequirements'
    _name = 'AtlasCREAMRequirements'
    _exportmethods = ['list_ce', 'list_se','list_sites','list_clouds', 'list_sites_cloud', 'cloud_from_sites', 'list_access_info', 'list_release_blacklist', 'getce' ]

    def __init__(self):
        super(AtlasCREAMRequirements,self).__init__()

    def merge(self,other):
        '''Merge requirements objects'''
      
        if not other: return self
      
        merged = AtlasCREAMRequirements()

        for name in self._schema.datadict.keys():
            try:
                attr = getattr(other,name)
            except AttributeError:
                attr = None
            if not attr: attr = getattr(self,name)
            setattr(merged,name,attr)
         
        return merged

    def convert(self):
        '''Convert the condition in a JDL specification'''
      
        requirements = []

        if self.sites:
            ce_requirement = ' ||\n     '.join([ 'other.GlueCEUniqueID == "%s"' % ce for ce in getCEsForSites(self.sites, self.excluded_sites, CREAM=True)])
            if not ce_requirement:
                raise BackendError('CREAM','Job cannot be submitted as no valid site has been specified.')
            requirements.append('( %s )' % ce_requirement)

        if self.os:
            os_name = self.os.lower()
            if os_name == 'slc3':
                requirements.append(slc3_req)
            elif os_name == 'slc4':
                requirements.append(slc4_req)
            else:
               raise BackendError('LCG','Job cannot be submitted as unknown OS %s has been requested.'%self.os)
            
        return requirements

    def getce(self):

        if self.cputime:
            mycputime = self.cputime
        else:
            mycputime = 0
        return getCEsForSites(self.sites, self.excluded_sites, CREAM=True, cputime=mycputime )

    def list_ce(self,ids):

        if isinstance(ids,str):
            ids = ids.split(':')

        return getCEsForSites(ids, CREAM=True)



from GangaCore.GPIDev.Lib.File.LocalFile import LocalFile

from GangaCore.GPIDev.Schema import Schema, Version, SimpleItem, ComponentItem

import GangaCore.Utility.logging
logger = GangaCore.Utility.logging.getLogger()

class SandboxFile(LocalFile):
    _schema = Schema(Version(1, 1), {'namePattern': SimpleItem(defvalue="", doc='pattern of the file name'),
                                     'localDir': SimpleItem(defvalue="", doc='local dir where the file is stored, used from get and put methods'),
                                     'subfiles': ComponentItem(category='gangafiles', defvalue=[], hidden=1, sequence=1, copyable=0, doc="collected files from the wildcard namePattern"),

                                     'compressed': SimpleItem(defvalue=False, typelist=[bool], protected=0, doc='wheather the output file should be compressed before sending somewhere')})
    _category = 'gangafiles'
    _name = "SandboxFile"

    def __init__(self, namePattern='', localDir='', **kwds):
        """ name is the name of the output file that is going to be processed
           in some way defined by the derived class
        """
        logger.warning(
            "SandboxFile is now deprecated please change your configuration to use LocalFile instead!")
        super(SandboxFile, self).__init__(namePattern, localDir, **kwds)



#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#

import os
import sys
import imp
import tempfile
import pickle
import subprocess
from GangaCore.GPIDev.Schema import Schema, Version, SimpleItem
from GangaCore.GPIDev.Base import GangaObject
from GangaCore.GPIDev.Adapters.IPostProcessor import PostProcessException
from GangaCore.GPIDev.Adapters.IMerger import IMerger
from GangaCore.Utility.Plugin import allPlugins
from GangaCore.Core.exceptions import GangaException
from GangaCore.GPIDev.Base.Proxy import GPIProxyObject

xml_schema = {}
xml_summary = {}

#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#


class GaudiXMLSummary(GangaObject):

    '''Class for handling XMLSummary objects.

    Obtaining the XML summary info requires the full path to the XML file
    and the location of the XMLSummaryBase version to use
    (store in env_var).

    One can instead simply initialize from a job object (which had a Gaudi
    application) as follows:

    gxml = GaudiXMLSummary(job,"summary.xml")
    sum = gxml.summary()

    N.b. the 2nd argument to the constructor defaults to "summary.xml".
    '''
    schema = {}
    schema['file'] = SimpleItem(defvalue='', typelist=['str'])
    schema['env_var'] = SimpleItem(defvalue='', typelist=['str'], hidden=0)
    _schema = Schema(Version(1, 1), schema)
    _category = ''
    _name = "GaudiXMLSummary"
    _exportmethods = ['create', 'summary']

    def __init__(self, job, file):
        super(GaudiXMLSummary, self).__init__()
        self.data = None
        self.create(job, file)

    def _attribute_filter__set__(self, n, v):
        if n == 'env_var':
            self.data = None
            return os.path.expanduser(v)
        else:
            return v

    def summary(self):
        '''Returns the summary object.'''
        try:
            data = self.data
            if not data:
                self._init()
        except:
            self._init()
        return self.data

    def _init(self):
        global xml_summary
        global xml_schema

        if not self.env_var:
            raise GangaException('XMLSummary env not set!')
        if not self.file:
            raise GangaException('File not specified!')
        if not os.path.exists(self.file):
            raise GangaException('%s does not exist!' % self.file)

        p = self._xmlPath()
        v = self.env_var
        if v not in xml_schema:
            if 'schema' in sys.modules:
                del sys.modules['schema']
            xml_schema[v] = imp.load_source('schema', p + '/schema.py')
            if 'summary' in sys.modules:
                del sys.modules['summary']
            xml_summary[v] = imp.load_source('summary', p + '/summary.py')
            xml_summary[v].__schema__ = xml_schema[v]

        sum = xml_summary[v].Summary(self._xmlSchema())
        sum.parse(self.file)
        self.data = sum

        if 'schema' in sys.modules:
            del sys.modules['schema']
        if 'summary' in sys.modules:
            del sys.modules['summary']
        return

    def create(self, job, file='summary.xml'):
        '''Sets up the GaudiXMLSummary object from a Ganga job.'''
        self._setEnvVar(job.application)
        self._setFile(job, file)
        self._init()

    def _setEnvVar(self, app):
        gaudi_env = app.getenv()
        self.env_var = gaudi_env['XMLSUMMARYBASEROOT']

    def _setFile(self, job, file):
        self.file = job.outputdir + file

    def _xmlPath(self):
        return self.env_var + '/python/XMLSummaryBase'

    def _xmlSchema(self):
        return self.env_var + '/xml/XMLSummary.xsd'

#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#


class GaudiXMLSummaryMerger(IMerger):

    '''Merger for XML summary files.'''
    _category = 'postprocessors'
    _exportmethods = ['merge']
    _name = 'GaudiXMLSummaryMerger'
    _schema = IMerger._schema.inherit_copy()
    _schema.datadict['env_var'] = SimpleItem(defvalue='', typelist=['str'])

    def mergefiles(self, file_list, output_file, jobs):
        from GangaCore.GPIDev.Lib.Job import Job
        from GangaCore.GPIDev.Base.Proxy import isType
        gaudi_env = {}
        if isType(jobs, Job):
            gaudi_env = jobs.application.getenv()
        elif len(jobs) > 0:
            gaudi_env = jobs[0].application.getenv()
        self.env_var = gaudi_env['XMLSUMMARYBASEROOT']
        # needed as exportmethods doesn't seem to cope with inheritance
        if not self.env_var:
            raise GangaException('XMLSummary env not set!')
        script_name = tempfile.mktemp('.py')
        script = open(script_name, 'w')
        dummy = GaudiXMLSummary()
        dummy.env_var = self.env_var

        # write py script
        script.write('import sys\n')
        script.write('sys.path.append("%s") \n' % dummy._xmlPath())
        script.write('import summary \n')
        script.write('sum = summary.Merge(%s,"%s") \n'
                     % (str(file_list), dummy._xmlSchema()))
        script.write('sum.write("%s") \n' % output_file)
        script.close()

        # run it
        proc = subprocess.Popen(['python', script_name])
        proc.wait()
        rc = proc.poll()
        if rc != 0:
            msg = 'Failed to merge XML summary file!'
            raise GangaException(msg)

        if not os.path.exists(output_file):
            raise GangaException('Failed to merge XML summary file!')

#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#

# Add it to the list of plug-ins
# allPlugins.add(_GaudiXMLSummaryMergeTool,'merge_tools',
#               '_GaudiXMLSummaryMergeTool')

#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#
